import unittest

from automon.integrations.splunk.config import SplunkConfig
from automon.integrations.splunk.client import SplunkClient

config_mock = SplunkConfig(
    host='localhost',
    username='user',
    password='pass')

config_cloud = SplunkConfig(
    host='splunkcloud.com',
    username='user',
    password='pass')

mock = SplunkClient(config_mock)
cloud = SplunkClient(config_cloud)


class SplunkConfigTest(unittest.TestCase):
    config = SplunkConfig()

    def test_init(self):
        self.assertIsNotNone(SplunkConfig())
        self.assertIsNotNone(self.config)

    def test_print(self):
        self.assertTrue(f'{self.config}')
        self.assertEqual(f'{config_mock}',
                         f'{config_mock.username}@{config_mock.scheme}://{config_mock.host}:{config_mock.port}')

    def test_info(self):
        self.assertTrue(self.config.info())
        self.assertEqual(config_mock.info(),
                         f'{config_mock.username}@{config_mock.scheme}://{config_mock.host}:{config_mock.port}')


class SplunkClientTest(unittest.TestCase):

    def test_client_connect(self):
        if cloud.connected:
            self.assertIsNotNone(cloud)
            self.assertIsNotNone(cloud.client)
            self.assertTrue(cloud.get_apps())
            self.assertTrue([x.name for x in cloud.get_apps()])

    def test_init(self):
        self.assertIsNotNone(SplunkClientTest())

    def test_jobs(self):
        if cloud.connected:
            self.assertTrue(cloud.jobs())
            self.assertTrue(cloud.job_summary())
            self.assertTrue(cloud.create_job('search * '))

    def test_search(self):
        if cloud.connected:
            self.assertTrue(cloud.search('search index=* | head 10'))

    def test_oneshot(self):
        if cloud.connected:
            self.assertIsNotNone(cloud.oneshot('search * | head 10'))


if __name__ == '__main__':
    unittest.main()



import os

from automon.log.logger import Logging
from automon.helpers.sanitation import Sanitation as S

log = Logging(name=__name__, level=Logging.DEBUG)


class Neo4jConfig:
    def __init__(self, user: str = None, password: str = None,
                 hosts: list = None, encrypted: bool = True):
        self._log = Logging(name=Neo4jConfig.__name__, level=Logging.ERROR)

        self.user = user or os.getenv('NEO4J_USER') or ''
        self.password = password or os.getenv('NEO4J_PASSWORD') or ''
        self.hosts = S.list_from_string(hosts) or \
                     S.list_from_string(os.getenv('NEO4J_SERVERS')) or []
        self.encrypted = encrypted

        if not self.user:
            self._log.error(f'NEO4J_USER not set')

        if not self.password:
            self._log.error(f'NEO4J_PASSWORD not set')

        if not self.hosts:
            self._log.error(f'NEO4J_SERVERS not set')



import os

import splunklib.binding as binding

from automon.log.logger import Logging


class SplunkConfig:
    def __init__(self, host: str = None, port: int = None, username: str = None,
                 password: str = None, verify: str = True, scheme: str = 'https',
                 app: NotImplemented = None, owner: NotImplemented = None,
                 token: str = None, cookie: str = None, timeout: int = 1):
        self._log = Logging(name=SplunkConfig.__name__, level=Logging.DEBUG)

        self.host = host or os.getenv('SPLUNK_HOST') or 'splunkcloud.com'
        self.port = port or os.getenv('SPLUNK_PORT') or 8090
        self.username = username or os.getenv('SPLUNK_USERNAME') or 'admin'
        self.password = password or os.getenv('SPLUNK_PASSWORD') or 'changeme'
        self.verify = verify
        self.scheme = scheme or 'https'
        self.app = app
        self.owner = owner
        self.token = token
        self.cookie = cookie
        self.handler = binding.handler(timeout=timeout)

    def info(self):
        return f'{self}'

    def __str__(self):
        return f'{self.username}@{self.scheme}://{self.host}:{self.port}'



import neo4j
import logging

from neo4j import GraphDatabase

from automon.log.logger import Logging
from automon.integrations.neo4j.cypher import Cypher
from automon.integrations.neo4j.config import Neo4jConfig
from automon.integrations.neo4j.helpers import Results
from automon.helpers.sanitation import Sanitation as S

logging.getLogger('neo4j').setLevel(logging.ERROR)
log = Logging(__name__, Logging.DEBUG)


class Neo4jClient:
    """Neo4j client"""

    def __init__(self, config: Neo4jConfig = None, user: str = None,
                 password: str = None, hosts: list or str = None,
                 encrypted: bool = None) -> neo4j:
        self._log = Logging(Neo4jClient.__name__, Logging.DEBUG)

        self.config = config or Neo4jConfig()
        self.user = user or self.config.user
        self.password = password or self.config.password
        self.hosts = S.list_from_string(hosts) or self.config.hosts
        self.encrypted = encrypted or self.config.encrypted

        for server in self.hosts:
            try:
                self.client = GraphDatabase.driver(
                    server, auth=(self.user, self.password),
                    encrypted=self.encrypted)

                self.driver = self.client
                self.connected = True
                log.info(f'Connected to neo4j server: {server}')
            except Exception as e:
                self.connected = False
                self._log.error(f'Cannot connect to neo4j server: {server}\t{e}',
                                enable_traceback=False)
        if not self.hosts:
            self.connected = False

    def __repr__(self):
        return f'{self.hosts}'

    def _send(self, cypher: str or Cypher) -> GraphDatabase.driver:
        """Send cypher to server"""

        if not self.connected:
            return False

        if isinstance(cypher, Cypher):
            cypher = f'{cypher}'

        with self.driver.session() as session:
            results = session.run(cypher)

        self._log.debug(f'{cypher}')
        self._log.debug(f'Results: {Results(results)}')

        return results

    def cypher(self, query: str):
        """Run a straight cypher query"""
        return self._send(S.strip(query))

    def delete_all(self):
        """Delete all nodes and relationships"""
        cypher = Cypher()
        cypher.delete_all()
        return self._send(cypher)

    def delete_node(self, prop: str, value: str, node: str = None):
        """Delete all matching nodes and its relationships"""
        cypher = Cypher()
        cypher.delete_node(prop=prop, value=value, node=node)
        return self._send(cypher)

    def merge(self, data: dict, label: str = None, node: str = None):
        """Merge a node"""

        # cypher_assertions = []
        # TODO: find a way to check if assertion has already been made
        # query = 'CREATE CONSTRAINT ON ( {node} {label} ) ASSERT {node}.raw IS UNIQUE'
        # query = query.format(node=node, label=label)
        # cypher_assertions.append(query)  # Pre cypher query

        cypher = Cypher()
        cypher.merge(node=node, label=label, data=data)

        # self._log.debug(f'{final_cypher}')

        return self._send(cypher)

    def create(self, data: dict, label: str = None, node: str = None):
        """Create a node"""

        cypher = Cypher()
        cypher.create(node=node, label=label, data=data)

        # self._log.debug(f'{final_cypher}')

        return self._send(cypher)

    def create_relationship(self, label: str, prop: str, value: str, other_prop: str,
                            other_value: str, relationship: str, other_label: str = None):
        """Create an A -> B relationship"""

        cypher = Cypher()
        cypher.relationship(
            label, prop, value, other_prop, other_value, relationship, other_label)

        self._log.debug(cypher)
        return self._send(cypher)

    def search(self, query: str):
        return



import os

from automon.log.logger import Logging

log = Logging(name=__name__, level=Logging.ERROR)


class ConfigSlack:
    slack_name = os.getenv('SLACK_USER') or ''
    slack_webhook = os.getenv('SLACK_WEBHOOK') or ''
    slack_proxy = os.getenv('SLACK_PROXY') or ''
    slack_token = os.getenv('SLACK_TOKEN') or ''

    SLACK_DEFAULT_CHANNEL = os.getenv('SLACK_DEFAULT_CHANNEL') or ''
    SLACK_INFO_CHANNEL = os.getenv('SLACK_INFO_CHANNEL') or ''
    SLACK_DEBUG_CHANNEL = os.getenv('SLACK_DEBUG_CHANNEL') or ''
    SLACK_ERROR_CHANNEL = os.getenv('SLACK_ERROR_CHANNEL') or ''
    SLACK_CRITICAL_CHANNEL = os.getenv('SLACK_CRITICAL_CHANNEL') or ''
    SLACK_WARN_CHANNEL = os.getenv('SLACK_WARN_CHANNEL') or ''
    SLACK_TEST_CHANNEL = os.getenv('SLACK_TEST_CHANNEL') or ''

    if not slack_token:
        log.error(f'SLACK_TOKEN not set')

    def __init__(self, slack_name: str = ''):
        self.slack_name = os.getenv('SLACK_USER') or slack_name or ''



import sys
import unittest

from automon.integrations.mac.airport.scanner import Airport


class AirportTest(unittest.TestCase):
    a = Airport()

    def test_Airport(self):
        if sys.platform == 'darwin':
            self.assertTrue(self.a.run())
            self.assertTrue(self.a.scan())
            self.assertTrue(self.a.scan(0))
            self.assertTrue(self.a.scan_summary())
            self.assertTrue(self.a.scan_summary(0))
            self.assertTrue(self.a.scan_xml())
            self.assertTrue(self.a.scan_xml(0))

            # self.assertTrue(self.a.set_channel(10))
            # self.assertTrue(self.a.disassociate())

            self.assertTrue(self.a.getinfo())
            self.assertTrue(self.a.create_psk(ssid='AAAAAAAA', passphrase='CALVIN'))


if __name__ == '__main__':
    unittest.main()



import os
import urllib3

from automon.log.logger import Logging


class MinioConfig:

    def __init__(self, endpoint: str = None, access_key: str = None, secret_key: str = None,
                 session_token: str = None, secure: bool = None, region: str = None,
                 http_client: urllib3.PoolManager = None):
        """"""
        self._log = Logging(name=MinioConfig.__name__, level=Logging.ERROR)

        self.endpoint = endpoint or os.getenv('MINIO_ENDPOINT')
        self.access_key = access_key or os.getenv('MINIO_ACCESS_KEY')
        self.secret_key = secret_key or os.getenv('MINIO_SECRET_KEY')
        self.session_token = session_token or os.getenv('MINIO_SESSION_TOKEN')
        self.secure = secure or os.getenv('MINIO_SECURE') or True
        self.region = region or os.getenv('MINIO_REGION')
        self.http_client = http_client or os.getenv('MINIO_HTTP_CLIENT')

        if not self.endpoint:
            self._log.error(f'missing MINIO_ENDPOINT')

        if not self.access_key:
            self._log.error(f'missing MINIO_ACCESS_KEY')

        if not self.secret_key:
            self._log.error(f'missing MINIO_SECRET_KEY')

    def __repr__(self):
        return f'{self.endpoint} {self.region}'


def use_public_server():
    return MinioConfig(
        endpoint='play.minio.io:9000',
        access_key='Q3AM3UQ867SPQQA43P2F',
        secret_key='zuf+tfteSlswRu7BJ86wekitnifILbZam1KYY3TG')



import io
import socket

from minio import Minio
from urllib.parse import urlparse

from automon.log.logger import Logging
from automon.integrations.minio.config import MinioConfig

log = Logging(name='minio', level=Logging.INFO)


class MinioClient:

    def __init__(self, config: MinioConfig = None):
        self._log = Logging(name=MinioClient.__name__, level=Logging.DEBUG)

        self.config = config or MinioConfig()

        check = urlparse(self.config.endpoint)
        if check_connection(check.hostname, check.port):
            self.client = Minio(
                endpoint=self.config.endpoint,
                access_key=self.config.access_key,
                secret_key=self.config.secret_key,
                session_token=self.config.session_token,
                secure=self.config.secure,
                region=self.config.region,
                http_client=self.config.http_client
            )
            self.connected = True
        else:
            self.connected = False

    def download_object(self, bucket, file):
        """ Minio object downloader
        """
        self._log.logging.debug(
            f'[downloader] Downloading: {bucket}/{file.object_name}')
        return self.client.get_object(bucket, file.object_name)

    def list_all_objects(self, bucket: str, folder: str = None, recursive: bool = True):
        """ List Minio objects
        """
        self._log.logging.debug(f'[list_all_objects] bucket: {bucket}, folder: {folder}')
        return self.client.list_objects(bucket, folder, recursive=recursive)

    def put_object(self, bucket_name: str, object_name: str, data: io.BytesIO, length: int = None,
                   content_type='application/octet-stream',
                   metadata=None, sse=None, progress=None,
                   part_size=None):
        """ Minio object uploader
        """

        self._log.debug(f'[{self.put_object.__name__}] Uploading: {object_name}')

        length = length or data.getvalue().__len__()

        try:
            put = self.client.put_object(bucket_name=bucket_name, object_name=object_name,
                                         data=data, length=length,
                                         content_type=content_type,
                                         metadata=metadata, sse=sse, progress=progress)
            self._log.info(
                f'[put_object] Saved to '
                f'{self.client._endpoint_url}/{bucket_name}/{object_name}'
            )

            return put

        except Exception as _:
            self._log.error(
                f'[{self.put_object.__name__}] Unable to save '
                f'{self.client._endpoint_url}/{bucket_name}/{bucket_name}'
            )

            return False

    def clear_bucket(self, bucket_name, folder=None):
        objects = self.list_all_objects(bucket_name, folder)
        for a in objects:
            print(a)
        return self.client.remove_objects(bucket_name, objects)

    def make_bucket(self, bucket_name):
        try:
            self.client.make_bucket(bucket_name)
            self._log.debug(f'[make_bucket] Created bucket: {bucket_name}')
            return True

        except Exception as _:
            self._log.debug(f'[make_bucket] Bucket exists: {bucket_name}')
            return False


def check_connection(host, port):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        s.settimeout(1)
        s.connect((host, port))
        s.close()
        return True
    except Exception as _:
        return False



from queue import Queue

import splunklib.results
import splunklib.client as client

from automon.log.logger import Logging
from automon.integrations.splunk.config import SplunkConfig
from automon.integrations.splunk.helpers import Job, Application

log = Logging(name=__name__, level=Logging.DEBUG)


class SplunkRestClient:

    def __int__(self, config: SplunkConfig = SplunkConfig()):
        self._log = Logging(SplunkRestClient.__name__, level=Logging.DEBUG)
        self.config = config


class SplunkClient:

    def __init__(self, config: SplunkConfig = None):
        self._log = Logging(SplunkClient.__name__, level=Logging.DEBUG)
        self.config = config or SplunkConfig()

        try:
            self.client = client.connect(
                host=self.config.host,
                port=self.config.port,
                username=self.config.username,
                password=self.config.password,
                verify=self.config.verify,
                scheme=self.config.scheme,
                app=self.config.app,
                owner=self.config.owner,
                token=self.config.token,
                cookie=self.config.cookie,
                handler=self.config.handler
            )

            # referred to as a service in docs
            self.service = self.client
            self.connected = True
            assert isinstance(self.service, client.Service)

        except Exception as e:
            self.connected = False
            self._log.error(f'{e}\t{self.config.host}:{self.config.port}', enable_traceback=False)

        self.queue = Queue()

    def info(self):
        return f'{self}'

    def search(self, query):
        """create normal search query"""
        kwargs_normalsearch = {"exec_mode": "normal"}
        job = self.create_job(query, **kwargs_normalsearch)
        # job = self.create_job(query)
        return self.results(job)

    def oneshot(self, query, earliest_time: str = '-15m', latest_time: str = 'now'):
        """create oneshot search"""

        kwargs_oneshot = {
            "earliest_time": earliest_time,
            "latest_time": latest_time
        }

        searchquery_oneshot = query
        oneshotsearch_results = self.service.jobs.oneshot(searchquery_oneshot, **kwargs_oneshot)

        reader = splunklib.results.ResultsReader(oneshotsearch_results)

        return [x for x in reader]

    def query(self, query):
        return self.search(query)

    def jobs(self):
        """get collection of search jobs"""
        return self.service.jobs

    def create_job(self, query, **kwargs):
        """create search job"""
        return self.service.jobs.create(query, **kwargs)

    @staticmethod
    def results(job: client.Job):
        """io blocking waiting for job results"""
        j = Job(job)
        while True:
            while not j.is_ready():
                pass
            if j.isDone == '1':
                break

        results = [x for x in splunklib.results.ResultsReader(j.results())]
        return results

    def job_summary(self):
        return f'There are {len(self.jobs())} jobs available to the current user'

    def get_apps(self):
        return [Application(x) for x in self.service.apps]

    def create_app(self, app_name):
        return self.service.apps.create(app_name)

    def get_app(self, app_name):
        return self.service.apps[app_name]

    def delete_app(self, app_name):
        return self.service.apps.delete(app_name)

    def app_info(self, app_name):
        return f'{self.service.apps[app_name]}'

    def __str__(self):
        if self.connected:
            return f'connected to {self.config}'
        return f'not connected to {self.config}'



import unittest

from automon.integrations.minio.client import MinioClient
from automon.integrations.minio.config import MinioConfig


class MinioTest(unittest.TestCase):
    def test_MinioConfig(self):
        self.assertTrue(MinioConfig())

    def test_MinioClient(self):
        self.assertTrue(MinioClient())


if __name__ == '__main__':
    unittest.main()



import requests
import elasticsearch

from datetime import datetime
from requests.auth import HTTPBasicAuth
from elasticsearch import Elasticsearch

from automon.log.logger import Logging
from automon.integrations.elasticsearch.config import ElasticsearchConfig
from automon.helpers.sanitation import Sanitation


class ElasticsearchClient(ElasticsearchConfig):
    def __init__(self, config: ElasticsearchConfig = ElasticsearchConfig()):
        self._log = Logging(ElasticsearchClient.__name__, Logging.DEBUG)

        self.config = config
        self.client = Elasticsearch(hosts=self.config.es_hosts,
                                    request_timeout=self.config.request_timeout,
                                    http_auth=self.config.http_auth,
                                    use_ssl=self.config.use_ssl,
                                    verify_certs=self.config.verify_certs,
                                    connection_class=self.config.connection_class)
        self.connected = self.client.ping() if self.config.es_hosts else False

        self.indices = []

    def rest(self, url: str) -> requests:
        try:
            if self.config.ELASTICSEARCH_USER and self.config.ELASTICSEARCH_PASSWORD:
                return requests.get(url, auth=HTTPBasicAuth(self.config.ELASTICSEARCH_USER,
                                                            self.config.ELASTICSEARCH_PASSWORD))
            else:
                return requests.get(url)

        except Exception as e:
            self._log.error(f'REST request failed: {e}')
            return False

    def ping(self):
        if self.connected:
            try:
                return self.client.ping()
            except Exception as e:
                self._log.error(f'Ping failed: {e}')
                return False

        return False

    def delete_index(self, index: str):
        if self.connected and index:
            try:
                return self.client.delete(index=Sanitation.ascii_numeric_only(index))
            except Exception as e:
                self._log.error(f'Delete index failed: {e}')
                return False

        return False

    # def delete_indices(self, index_pattern):
    #     """Requires user interaction"""
    #
    #     if not self.connected:
    #         return False
    #
    #     retrieved_indices = [x for x in self.client.search_indices(index_pattern).keys()]
    #     num_indices = len(retrieved_indices)
    #
    #     if not num_indices:
    #         self._log.debug(f'No indices found')
    #         return False
    #
    #     self._log.info(f'Search found {num_indices} indices')
    #
    #     for index in retrieved_indices:
    #         self._log.debug(index)
    #
    #     # TODO: Find a way to undo index deletions
    #     #       One way could be to rename the indices and store a link to the new
    #     #       indices in a node of deleted indices
    #     if num_indices < 2:
    #         msg = f"\nYOU'RE ABOUT TO DELETE {num_indices} INDEX! ARE YOU SURE YOU WANT TO CONTINUE? "
    #     elif num_indices > 1:
    #         msg = f"\nYOU'RE ABOUT TO DELETE {num_indices} INDICES! ARE YOU SURE YOU WANT TO CONTINUE? "
    #     msg += 'THIS CANNOT BE UNDONE! DECIDED WISELY [y/N]'
    #     print(msg)
    #
    #     answer = str(input()).lower()
    #
    #     if not answer:
    #         answer = 'N'
    #
    #     if answer == 'y':
    #         for index in retrieved_indices:
    #             msg = f'Deleting {index}...'
    #             print(msg, end='')
    #             # Delete the index
    #             self.delete_index(index)
    #             print('done')
    #     else:
    #         print('Whew, you might have just blew it, if you had said yes')

    def search_indices(self, index_pattern):
        if self.connected and index_pattern:
            try:
                retrieved_indices = self.client.indices.get(index_pattern)
                num_indices = len(retrieved_indices)
                self._log.info(f'Search found {num_indices} indices')
                return retrieved_indices
            except elasticsearch.exceptions.NotFoundError:
                self._log.error(
                    f"You provided the index pattern '{index_pattern}', but returned no results")
            except Exception as e:
                self._log.error(f'Failed to search indices: {e}')

        return False

    def get_indices(self):
        if self.connected:
            try:
                retrieved_indices = self.client.indices.get('*')
                self.indices.extend(retrieved_indices)
                self._log.info(f'Retrieved {len(retrieved_indices)} indices')
                for i in retrieved_indices:
                    info = retrieved_indices.get(i)
                    date = int(info.get('settings').get('index').get('creation_date')) / 1000.0
                    date = datetime.fromtimestamp(date).strftime("%A, %B %d, %Y %I:%M:%S")
                    self._log.debug(f'Index: (created: {date})\t{i}')
                return True
            except Exception as e:
                self._log.error(f'Failed to get indices: {e}')

        return False



import os
import logging

from elasticsearch import RequestsHttpConnection

from automon.log.logger import Logging
from automon.helpers.sanitation import Sanitation as S

logging.getLogger('elasticsearch').setLevel(logging.ERROR)
logging.getLogger('urllib3.connectionpool').setLevel(logging.ERROR)


class ElasticsearchConfig:
    def __init__(self, endpoints: str = None, proxy=None,
                 request_timeout: int = 1,
                 http_auth: tuple = None,
                 use_ssl: bool = True,
                 verify_certs: bool = True,
                 connection_class: RequestsHttpConnection = RequestsHttpConnection):
        self._log = Logging(ElasticsearchConfig.__name__, Logging.DEBUG)

        hosts = S.list_from_string(endpoints) or \
                S.list_from_string(os.getenv('ELASTICSEARCH_HOSTS')) or None
        # hosts = [{'host': x} for x in hosts]
        self.es_hosts = hosts
        self.ELASTICSEARCH_HOSTS = self.es_hosts
        self.ELASTICSEARCH_USER = os.getenv('ELASTICSEARCH_USER') or ''
        self.ELASTICSEARCH_PASSWORD = os.getenv('ELASTICSEARCH_PASSWORD') or ''

        self.es_proxy = proxy
        self.request_timeout = request_timeout

        if self.ELASTICSEARCH_USER and self.ELASTICSEARCH_PASSWORD:
            self.http_auth = (self.ELASTICSEARCH_USER, self.ELASTICSEARCH_PASSWORD)
        else:
            self.http_auth = http_auth

        self.use_ssl = use_ssl
        self.verify_certs = verify_certs
        self.connection_class = connection_class

        if not self.es_hosts:
            self._log.error(f'Missing ELASTICSEARCH_HOSTS')

    def __eq__(self, other):
        if not isinstance(other, ElasticsearchConfig):
            self._log.error(f'Not implemented')
            return NotImplemented

        return self.es_hosts == other.es_hosts


class SnapshotBot:
    def __init__(self):
        self.slack_name = 'Elasticsearch Daily Monitor bot'
        self.es_hosts = ElasticsearchConfig().es_hosts
        self.es_repository = os.getenv('ELASTICSEARCH_REPOSITORY')
        self.snapshots_prefix = os.getenv('SNAPSHOTS_PREFIX')


class JVMBot:
    def __init__(self):
        self.slack_name = 'Elasticsearch JVM Monitor bot'
        self.es_hosts = ElasticsearchConfig().es_hosts



import datetime


class Dates:
    today = f'{datetime.datetime.today()}'
    now = f'{datetime.datetime.now()}'
    iso = f'{datetime.datetime.isoformat(datetime.datetime.now())}'
    iso_short = f'{datetime.date.today().isoformat()}'

    filename_timestamp = str(datetime.datetime.now().isoformat()).replace(':', '_')



import re

from urllib.parse import urlencode
from datetime import datetime, timezone

from automon.log.logger import Logging

log = Logging(__name__, Logging.DEBUG)


class Cypher:

    def __init__(self):
        self.cypher = []
        self._node = 'Node'

    def __repr__(self):
        return self.consolidate()

    def start(self, data: dict, node: str = None, label: str = ''):
        """alias to merge"""
        return self.merge(node=node, label=label, data=data)

    def merge(self, data: dict, node: str = None, label: str = ''):
        """Merge a node

        MERGE (Node :`human` { `name`: "finn" })
        ON CREATE SET Node.first_seen = "2021-02-25T03:19:47.438596+00:00"
        ON CREATE SET Node.first_seen_ts = timestamp()
        SET Node.last_seen = "2021-02-25T03:19:47.438901+00:00"
        SET Node.last_seen_ts = timestamp()
        RETURN *

        """
        node = node or self._node
        label = self.assert_label(label)
        self.cypher.append(f'MERGE ({node} {label} {{')
        self.dict_to_cypher(data)
        self._end()
        self.first_seen()
        self.last_seen()
        self._return_asterisk()

    def create(self, data: dict, node: str = None, label: str = ''):
        """Create a node"""
        node = node or self._node
        label = self.assert_label(label)
        self.cypher.append(f'CREATE ({node} {label} {{')
        self.dict_to_cypher(data)
        self._end()
        self.last_seen()
        self._return_asterisk()

    def unique(self, node: str = None, label: str = ''):
        node = node or self._node
        label = self.assert_label(label)
        self.cypher.append(f'UNIQUE ({node} {label} {{')

    def match(self, node: str, prop: str, value: str):
        """Match a node"""
        node = node or self._node
        # MATCH (n {name: 'Andy'})
        self.cypher.append(f'MATCH ({node} {{{prop}: "{value}"}})')

    def relationship(self, label: str, prop: str, value: str, other_prop: str,
                     other_value: str, relationship: str, other_label: str = None):
        """Create an A -> B relationship

        MATCH (a:Person), (b:Person)
        WHERE a.name = 'A' AND b.name = 'B'
        CREATE (a)-[r:RELTYPE]->(b)
        RETURN type(r)

        """

        label = self.assert_label(label)
        other_label = self.assert_label(other_label)
        relationship = self.assert_label(relationship)

        if not other_label:
            other_label = label

        self.cypher.append(f'MATCH (a{label}), (b{other_label})')
        self.cypher.append(
            f'WHERE a.{prop} = "{value}" AND b.{other_prop} = "{other_value}"')
        self.cypher.append(f'MERGE (a)-[r{relationship}]->(b)')
        self.cypher.append(f'RETURN type(r)')

    def _end(self):
        """End of cypher"""
        self.cypher.append('})')

    def _return_asterisk(self):
        self.cypher.append('RETURN *')

    def first_seen(self, node: str = None):
        """Node first_seen property"""
        node = node or self._node
        time = datetime.now(tz=timezone.utc).isoformat()
        # ON CREATE SET keanu.created = timestamp()
        self.cypher.append(f'ON CREATE SET {node}.first_seen = "{time}"')
        self.cypher.append(f'ON CREATE SET {node}.first_seen_ts = timestamp()')

    def last_seen(self, node: str = None):
        """Node last_seen property"""
        node = node or self._node
        time = datetime.now(tz=timezone.utc).isoformat()
        self.cypher.append(f'SET {node}.last_seen = "{time}"')
        self.cypher.append(f'SET {node}.last_seen_ts = timestamp()')

    def on_create_set(self):
        # timestamp of first seen
        # query.append('ON CREATE SET {}.timestamp = "{}"'.format(node, timestamp_date))
        return

    # @staticmethod
    # def begin():
    #     return '{'

    def dict_to_cypher(self, obj: dict):
        """Dict to cypher

        must be { `key`: 'value' }
        """

        obj = self.prepare_dict(obj)
        cypher = []
        i = 1
        for key in obj.keys():
            value = obj.get(key)
            query = f'`{key}`: "{value}"'

            if i < len(obj.keys()):
                cypher.append(query + ',')
            else:
                cypher.append(query)
            i += 1

        self.cypher.append(' '.join(cypher))

    def consolidate(self) -> str:
        """Join cypher queries list into a string"""
        return ' '.join(self.cypher).strip()

    @staticmethod
    def prepare_dict(blob: dict) -> dict:
        """All inputs first needs to dicts"""
        try:
            return dict(blob)
        except Exception as _:
            return dict(raw=urlencode(blob))

    @staticmethod
    def assert_label(label: str) -> str:
        """Make sure neo4j label is formatted correctly"""

        if not label:
            return ''

        if re.search('[:]', label):
            log.error(f"Invalid label '{label}': Remove the colon from the label")
            label = label.replace(':', '')

        if not re.search('[a-zA-Z]', label[0]):  # First letter of a label must be a letter
            log.error(f"Invalid label '{label}': First character of Neo4j :LABEL must be a letter")
        else:
            return f':`{label}`'  # :`Label`

    def delete_all(self):
        """Delete all nodes and relationships"""
        self.cypher.append(f'MATCH (n) DETACH DELETE n')

    def delete_node(self, prop: str, value: str, node: str = None):
        """Delete all matching nodes and its relationships"""
        node = node or self._node
        # MATCH (n {name: 'Andy'})
        # DETACH DELETE n
        self.match(prop=prop, value=value, node=node)
        self.cypher.append(f'DETACH DELETE {node}')



import sys
import xmltodict
import subprocess

from queue import Queue
from subprocess import PIPE

from automon.log.logger import Logging
from automon.helpers.dates import Dates
from automon.integrations.mac.airport.helpers import Scan

flags = {
    '-s': 'scan for wireless networks',
    '-c': '--channel=[<arg>]    Set arbitrary channel on the card',
    '-I': '--getinfo            Print current wireless status, e.g. signal info, BSSID, port type etc.',
    '-z': '--disassociate       Disassociate from any network',
    '-s': '--scan=[<arg>]       Perform a wireless broadcast scan. Will perform a directed scan if the optional <arg> is provided',
    '-x': '--xml                Print info as XML',
    '-P': '--psk                Create PSK from specified pass phrase and SSID.',

}


class Airport:
    def __init__(self):
        """Apple Airport Wi-Fi Utility

        A long hidden airport command line utility buried deep in Mac OS X can be used to scan for and find
        available wireless networks. This powerful tool is very helpful for network admins and systems
        administrators, but it’s handy for the average user to help discover nearby wi-fi routers as well.

        Tested on:
          - Apple M1


        # makbuk: eric$ /System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport  -h
        # Supported arguments:
        #  -c[<arg>] --channel=[<arg>]    Set arbitrary channel on the card
        #  -z        --disassociate       Disassociate from any network
        #  -I        --getinfo            Print current wireless status, e.g. signal info, BSSID, port type etc.
        #  -s[<arg>] --scan=[<arg>]       Perform a wireless broadcast scan.
        # 				   Will perform a directed scan if the optional <arg> is provided
        #  -x        --xml                Print info as XML
        #  -P        --psk                Create PSK from specified pass phrase and SSID.
        # 				   The following additional arguments must be specified with this command:
        #                                   --password=<arg>  Specify a WPA password
        #                                   --ssid=<arg>      Specify SSID when creating a PSK
        #  -h        --help               Show this help
         """

        self._log = Logging(name=Airport.__name__, level=Logging.DEBUG)

        self._airport = '/System/Library/PrivateFrameworks/Apple80211.framework/Versions/Current/Resources/airport'

        self.is_mac = False

        self._connected = None
        self._channel = None
        self._network = None
        self._info = None

        self.scans = []
        self.parsed_scans = []
        self.ssids = []

        self._queue = Queue()

        if sys.platform == 'darwin':
            self.is_mac = True
        else:
            self._log.error(f'Platform is not a Mac! ({sys.platform})')

    def __repr__(self):
        return ''

    @staticmethod
    def _command(command: str) -> list:
        return f'{command}'.split(' ')

    def run(self, args=''):
        command = self._command(f'{self._airport} {args}')

        call = subprocess.Popen(command, stdin=PIPE, stdout=PIPE, stderr=PIPE, text=True)
        output, errors = call.communicate()
        call.wait()

        result = {
            'scan_date': Dates.iso,
            'cmd': command,
            'output': output,
            'errors': errors
        }

        return result

    def scan(self, channel: int = None, args: str = ''):
        if channel:
            result = self.run(args=f'-s{channel} {args}')
        else:
            result = self.run(args=f'-s {args}')

        self.scans.append(result)

        return result

    def scan_summary(self, channel: int = None, args: str = '', output: bool = True):
        if channel is not None:
            res = self.scan(channel=channel, args=args)
        else:
            res = self.scan(args=args)

        if not channel:
            self._log.debug(f'Channel: Any')
        else:
            self._log.debug(f'Channel: {channel}')

        if output:
            print(f"{res['output']}")
            self._log.debug(f"\n{res['output']}")

        return res

    def set_channel(self, channel: int):
        return self.run(args=f'-c{channel}')

    def getinfo(self):
        return self.run(args='-I')

    def disassociate(self):
        return self.run(args='-z')

    def create_psk(self, ssid: str, passphrase: str):
        return self.run(args=f'-P --ssid={ssid} --password={passphrase}')['output']

    def scan_xml(self, channel: int = None):

        while True:

            if channel is not None:
                scan = self.scan(args='-x', channel=channel)
            else:
                scan = self.scan(args='-x')

            data = scan['output']
            data = [x.strip() for x in data.splitlines()]
            data = ''.join(data)

            try:
                root = xmltodict.parse(data)
                break
            except Exception as e:
                self._log.error(f'Scan not parsed: {e}, {scan["cmd"]}')

        parsed = Scan(scan=scan, result=root)

        if not parsed:
            self._log.error(f'root not parsed')

        self.parsed_scans.append(parsed)

        for ssid in parsed.ssids:
            self._queue.put(ssid)
            self.ssids.append(ssid)

        self.ssids.sort()

        return parsed



import unittest

from automon.integrations.neo4j.client import Neo4jClient
from automon.integrations.neo4j.config import Neo4jConfig
from automon.integrations.neo4j.cypher import Cypher


class Neo4jTest(unittest.TestCase):
    client = Neo4jClient(Neo4jConfig(encrypted=False))

    def test_Neo4jClient(self):
        self.assertIsNotNone(Neo4jClient(hosts='bolt://localhost:0', encrypted=False))

    def test_cypher(self):

        if self.client.connected:
            # self.assertTrue(self.client.delete_all())
            self.assertTrue(self.client.merge(label='human', data={'name': 'finn'}))
            self.assertTrue(self.client.merge(label='dog', data={'name': 'jake', 'magic': True}))
            self.assertTrue(self.client.merge(label='dog', data={'name': 'jake'}))
            self.assertTrue(self.client.merge(label='math', data={'results': 1}))
            self.assertTrue(self.client.merge(label='big dict', data={'isthat': 'new', 'no': "it's not"}))
            self.assertTrue(self.client.merge(data={'no': 'labels', 'look': 'mom'}))
            # self.assertTrue(self.client.delete_all())

    def test_relationships(self):
        if self.client.connected:
            self.assertTrue(self.client.merge(label='bubble tea', data={'type': 'milk tea'}))
            self.assertTrue(self.client.merge(label='bubble tea', data={'type': 'earl grey'}))

            self.assertTrue(self.client.create_relationship(
                'bubble tea',
                'type', 'milk tea',
                'type', 'earl grey',
                'better than'))

    def test_delete_node(self):
        if self.client.connected:
            self.assertTrue(self.client.create(label='orange', data={'type': 'juice'}))
            self.assertTrue(self.client.create(label='orange', data={'flavor': 'orange'}))

            self.assertTrue(self.client.delete_node(prop='type', value='juice'))
            self.assertTrue(self.client.delete_node(prop='flavor', value='orange'))

            self.assertTrue(self.client.create(label='orange', data={'type': 'juice'}))
            self.assertTrue(self.client.create(label='orange', data={'flavor': 'orange'}))

    def test_assert_label(self):
        test = Cypher()

        self.assertEqual(test.assert_label(':test'), ':`test`')
        self.assertIsNone(test.assert_label('1test'))
        self.assertEqual(test.assert_label('works'), ':`works`')


if __name__ == '__main__':
    unittest.main()



import unittest

from automon.integrations.slack.client import SlackClient, SlackError, BotInfo
from automon.integrations.slack.slack_logger import AsyncSlackLogging
from automon.integrations.slack.config import ConfigSlack
from automon.integrations.slack.slack_formatting import Format, Chat, Emoji


class SlackTest(unittest.TestCase):
    def test_ConfigSlack(self):
        self.assertTrue(ConfigSlack())

    def test_Slack(self):
        self.assertTrue(SlackClient())

    def test_BotInfo(self):
        bot = {
            "ok": True,
            "bot": {
                "id": "B061F7JD2",
                "deleted": False,
                "name": "beforebot",
                "updated": 1449272004,
                "app_id": "A161CLERW",
                "user_id": "U012ABCDEF",
                "icons": {
                    "image_36": "https://...",
                    "image_48": "https://...",
                    "image_72": "https://..."
                }
            }
        }

        self.assertTrue(BotInfo)
        self.assertTrue(BotInfo(bot))
        self.assertEqual(BotInfo(bot), BotInfo(bot))
        self.assertNotEqual(BotInfo(bot), bot)
        self.assertTrue(f'{BotInfo(bot)}')

    def test_SlackError(self):
        self.assertTrue(SlackError)

    def test_SlackLogging(self):
        test = AsyncSlackLogging(username='automonbot')

        self.assertTrue(test)
        self.assertIsNone(test.info('info'))
        self.assertIsNone(test.warn('warn'))
        self.assertIsNone(test.debug('debug'))
        self.assertIsNone(test.error('error'))
        self.assertIsNone(test.critical('critical'))
        self.assertIsNone(test.test('test'))
        self.assertIsNone(test.close())

    def test_Format(self):
        self.assertTrue(Format.blockquote)
        self.assertTrue(Format.codeblock)

    def test_Chat(self):
        self.assertTrue(Chat.Format('test', Format.codeblock))
        self.assertTrue(Chat.wrap('test', Format.blockquote))
        self.assertFalse(Chat.none())
        self.assertTrue(Chat.clean('test'))
        self.assertFalse(Chat.clean(None))
        self.assertTrue(Chat.string('test'))
        self.assertFalse(Chat.string(None))
        self.assertFalse(Chat.multi_to_single_line(None))
        self.assertFalse(Chat.multi_to_single_line('\n'))
        self.assertTrue(Chat.multi_to_single_line('test'))
        self.assertTrue(Chat.multi_to_single_line('test\n'))
        self.assertTrue(Chat.multi_to_single_line('test\ntest'))

    def test_Emoji(self):
        self.assertTrue(Emoji.warning)
        self.assertTrue(Emoji.information_source)
        self.assertTrue(Emoji.magnifying_glass)
        self.assertTrue(Emoji.sos)
        self.assertTrue(Emoji.waiting)
        self.assertTrue(Emoji.still_waiting)
        self.assertTrue(Emoji.grey_question)
        self.assertTrue(Emoji.questionbang)
        self.assertTrue(Emoji.skull)
        self.assertTrue(Emoji.skull_and_crossbones)
        self.assertTrue(Emoji.file)
        self.assertTrue(Emoji.announcement)
        self.assertTrue(Emoji.yay)

# if __name__ == '__main__':
#     unittest.main()



import os

from automon.log.logger import Logging


class SwiftConfig:
    OS_USERNAME = os.getenv('OS_USERNAME') or ''
    OS_PASSWORD = os.getenv('OS_PASSWORD') or ''
    OS_AUTH_URL = os.getenv('OS_AUTH_URL') or ''
    OS_PROJECT_ID = os.getenv('OS_PROJECT_ID') or ''
    OS_PROJECT_NAME = os.getenv('OS_PROJECT_NAME') or ''
    OS_USER_DOMAIN_NAME = os.getenv('OS_USER_DOMAIN_NAME') or ''
    OS_PROJECT_DOMAIN_ID = os.getenv('OS_PROJECT_DOMAIN_ID') or ''
    OS_REGION_NAME = os.getenv('OS_REGION_NAME') or 'RegionOne'
    OS_INTERFACE = os.getenv('OS_INTERFACE') or 'public'
    OS_IDENTITY_API_VERSION = os.getenv('OS_IDENTITY_API_VERSION') or '3'
    SWIFTCLIENT_INSECURE = os.getenv('SWIFTCLIENT_INSECURE') or 'True'

    def __init__(self):
        self.log = Logging(name=SwiftConfig.__name__, level=Logging.DEBUG)

        if not self.OS_USERNAME:
            self.log.error(f'missing OS_USERNAME')
        if not self.OS_PASSWORD:
            self.log.error(f'missing OS_PASSWORD')
        if not self.OS_AUTH_URL:
            self.log.error(f'missing OS_AUTH_URL')
        if not self.OS_PROJECT_ID:
            self.log.error(f'missing OS_PROJECT_ID')
        if not self.OS_PROJECT_NAME:
            self.log.error(f'missing OS_PROJECT_NAME')
        if not self.OS_USER_DOMAIN_NAME:
            self.log.error(f'missing OS_USER_DOMAIN_NAME')
        if not self.OS_PROJECT_DOMAIN_ID:
            self.log.error(f'missing OS_PROJECT_DOMAIN_ID')
        if not self.OS_REGION_NAME:
            self.log.error(f'missing OS_REGION_NAME')
        if not self.OS_INTERFACE:
            self.log.error(f'missing OS_INTERFACE')
        if not self.OS_IDENTITY_API_VERSION:
            self.log.error(f'missing OS_IDENTITY_API_VERSION')
        if not self.SWIFTCLIENT_INSECURE:
            self.log.error(f'missing SWIFTCLIENT_INSECURE')

    def __eq__(self, other):
        if not isinstance(other, SwiftConfig):
            self.log.error(f'Not implemented')
            return NotImplemented

        return self.OS_USERNAME == other.OS_USERNAME and \
               self.OS_PASSWORD == other.OS_PASSWORD and \
               self.OS_AUTH_URL == other.OS_AUTH_URL and \
               self.OS_PROJECT_ID == other.OS_PROJECT_ID and \
               self.OS_PROJECT_NAME == other.OS_PROJECT_NAME and \
               self.OS_PROJECT_DOMAIN_ID == other.OS_PROJECT_DOMAIN_ID



from automon.log.logger import Logging

log = Logging(name=__name__, level=Logging.DEBUG)


class Scan:
    def __init__(self, scan: dict, result: dict):
        self._log = Logging(name=Scan.__name__, level=Logging.DEBUG)
        self._data = result
        self.scan_date = scan['scan_date']
        self.ssids = []

        try:
            self._scans = result['plist']['array']['dict']
        except Exception as e:
            self.ssids = []
            self._log.debug(f'No ssids found, {" ".join(scan["cmd"])}')

        self.ssids = [Ssid(ssid) for ssid in self._scans]

        self.summary = {}

        for ssid in self.ssids:
            s_ssid = ssid.ssid
            self.summary[s_ssid] = 0

        for ssid in self.ssids:
            s_ssid = ssid.ssid
            s_mac = ssid.mac
            s_device = ssid.device_info

            self.summary[s_ssid] += 1

        sort = sorted(self.summary.items(), key=lambda kv: kv[1], reverse=True)

        [self._log.debug(f'{k}: {v}')
         for k, v in sort]

        self._log.debug(f'Total SSIDs: {len(self.ssids)}')

    def __repr__(self):
        return f'{[[x.ssid for x in self.ssids]]}'

    def __eq__(self, other):
        if isinstance(other, Scan):
            return self.ssids == other.ssids


class Ssid:
    def __init__(self, ssid: dict):
        self._log = Logging(name=Ssid.__name__, level=Logging.DEBUG)
        self._ssid = ssid

        self.mac = ssid['string'][0]
        self.ssid = ssid['string'][1]
        self.device_info = self._device_info(ssid) or ''

        log.debug(f'Found SSID: {self.ssid} ({self.mac}) {self.device_info}')

    @staticmethod
    def _device_info(ssid):
        if 'dict' in ssid:
            for key in ssid['dict']:
                if isinstance(key, dict):
                    if 'string' in key.keys() and 'data' in key.keys() and 'dict' in key.keys():
                        if isinstance(key['string'], list):
                            return key['string']

    def __repr__(self):
        return f'{self.ssid} {self.mac} {self.device_info}'

    def __eq__(self, other):
        if isinstance(other, Ssid):
            return self.mac == other.mac

    def __lt__(self, other):
        if isinstance(other, Ssid):
            return self.ssid < other.ssid



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="automonisaur",
    version="0.0.19",
    author="naisanza",
    author_email="naisanza@gmail.com",
    description="Core libraries for automonisaur",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/TheShellLand/automon-core",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.8',
    install_requires=[
        'Flask-Login>=0.5.0',
        'Flask-WTF>=0.14.3',
        'Flask>=1.1.2',
        'bcrypt>3.1.7',
        'coverage>=5.4',
        'elasticsearch>=7.11.0',
        'neo4j-driver>=4.2.1',
        'nest-asyncio>=1.5.1',
        'pytest-cov>=2.11.1',
        'pytest>=6.2.2',
        'python-keystoneclient>=4.2.0',
        'python-swiftclient>=3.11.0',
        'pytz>=2021.1',
        'requests>=2.25.1',
        'slackclient>=2.9.3',
        'splunk-sdk>=1.6.15',
        'xmltodict>=0.12.0'
    ]
)



import socket
from urllib.parse import urlparse


class Networking:

    @staticmethod
    def check_connection(url):
        endpoint = urlparse(url)

        host = endpoint.hostname
        port = endpoint.port

        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            s.settimeout(1)
            s.connect((host, port))
            s.close()
            return True
        except Exception as _:
            return False

    @staticmethod
    def urlparse(url):
        return urlparse(url)



import re

from neo4j.work.result import Result

from automon.log.logger import Logging

log = Logging(name=__name__, level=Logging.DEBUG)


class Results(Result):
    def __init__(self, results):
        self._results = results

        try:
            self._summary = results._summary
            self.counters = self._summary.counters
            self.notifications = self._summary.notifications
            self.query = self._summary.query
            self.query_type = self._summary.query_type

            self.metadata = self._summary.metadata
            self.stats = self.metadata.get('stats', None)
        except Exception as e:
            log.error(e, enable_traceback=False)

    def __str__(self):
        try:
            return f'{self.stats}'
        except:
            return 'no nodes'
